#include "softwareTransport.h"
#include "abstractElement.h"
#include "beam2Moments.h"

#include <algorithm>


softwareTransport::softwareTransport() : abstractSoftware()
{
  ;
}

softwareTransport::softwareTransport(globalParameters* globals, elementsCollection* beamLine) : abstractSoftware(globals, beamLine)
{
  ;
}


bool softwareTransport::createInputFile(particleBeam* beamBefore, unsigned int numeroDeb, unsigned int numeroFin, string workingDir)
{
  unsigned int k;
  if ( numeroDeb < 1 || numeroFin > beamLinePtr_->size() ) {
    cerr << " numero of element out of limits " << endl;
    return false;
  }

  unsigned indexDeb = numeroDeb - 1;
  unsigned indexFin = numeroFin - 1;

  abstractElement* premierEl = NULL;
  premierEl = beamLinePtr_->getElementPointerFromIndex(indexDeb);

  // ceinture et bretelles
  if ( premierEl == NULL ) {
    cerr << " premier element inexistant ?? " << endl;
    return false;
  }


  ofstream outfile;
  string name = workingDir + "transport.input";
  outfile.open(name.c_str(), ios::out);
  if (!outfile) 
    {
      cerr << " error opening output stream " << name << endl;
      return false;
    }

  outfile << string("'titre provisoire'") << endl;
  outfile << " 0 " << endl;
  outfile << " UTRANS " << endl;


  // si le premier element a traiter est un 'beam', on ne fait rien pour l'instant
  // il sera traite plus bas de maniere standard

  // Dans le cas où le premier element n'est pas beam
  if ( premierEl->getNomdElement().getElementType() != beam ) {
    // s'il n'y a pas faisceau courant, erreur
    if ( beamBefore == NULL ) {
      cout << " pas de faisceau " << endl;
      return false;
    }
    else {
      // s'il y a un faisceau courant
      if ( !beamBefore->momentRepresentationOk() ) {
	// s'il n'est pas au format transport (issu de parmela ?)
	// on le met au format transport, s'il est deja au format 'particules' (par ex/ parmela)
	if ( !beamBefore->particleRepresentationOk() ) {
	  cout << " ca cafouille dans les representations de faisceau : ni particules, ni moments ? " << endl;
	  return false;
	}
	else {
	  beamBefore->buildMomentRepresentation();
	}
      }
      // ici, en principe, on a un faisceau correct
      // on ecrit sur le fichier input
      const beam2Moments& moments = beamBefore->getTransportMoments();
      double P0 = beamBefore->getP0Transport();
      outfile << moments.writeToTransportInput(P0);
    }
  }



  outfile << " PRINT, BEAM, ON; " << endl;


  abstractElement* elPtr;
  for ( k=indexDeb; k <= indexFin; k++)
    {
      elPtr = beamLinePtr_->getElementPointerFromIndex(k);
      outfile << elPtr->transportOutputFlow() << endl;
    }

  outfile << " SENTINEL " << endl;
  outfile << " SENTINEL " << endl;

  outfile.close();
  return true;
}



bool softwareTransport::execute( unsigned int numeroDeb, unsigned int numeroFin, string workingDir, string& resul)
{
  ostringstream sortie;
  resul.clear();
  sortie << " EXECUTION DE TRANSPORT DE " << numeroDeb << " a " << numeroFin << endl;
  // if ( !createInputFile( &beamAfterElement.back(), numeroDeb, numeroFin, workingDir) )
  //   {
  //     sortie << " error creating transport input file "  << endl;
  //     resul = sortie.str();
  //     return false;
  //   }

  bool ExecuteStatus = true;
  string bufString;
  char buf[132];
  string transportJob = workingDir + "transport";
  transportJob += string(" <  ");
  transportJob += workingDir + "transport.input";
  cout << " job transport= " << transportJob << endl;

  ofstream outfile;
  string nameOut = workingDir + "transport.output";
  string resultOfRun;
  bool success = launchJob(transportJob, resultOfRun);
  sortie << resultOfRun << endl;
  if ( !success) {
    sortie << " launching of transport failed " << endl;
    ExecuteStatus = false;
  }
  else {
    cout << " execution transport MARCHE " << endl;
    outfile.open(nameOut.c_str(), ios::out);
    if (!outfile) 
      {
	sortie << " error first opening transport output stream " << nameOut << endl;
	ExecuteStatus = false;
      }
    else {
      // on copie la sortie dans un fichier 'transport.out'
      outfile << resultOfRun << endl;
      outfile.close();
      // ExecuteStatus = buildBeamAfterElements(numeroDeb,numeroFin, beamAfterElement, workingDir);
      // if ( !ExecuteStatus ) {
      // 	sortie << " reading transport results  failed " << endl;
      //      }
    }
  }
  resul =  sortie.str();
  return ExecuteStatus;
}


bool softwareTransport::buildBeamAfterElements(unsigned int numeroDeb,unsigned int numeroFin, vector<particleBeam>& beams, string workingDir) {

  bool result = true;
unsigned k;
	for ( k= numeroDeb; k <= numeroFin; k++)
	  {
	    beams.push_back(particleBeam());
	    //	    string elemLabel = getLabelFromElementNumero (k);

	    abstractElement* ptr = beamLinePtr_->getElementPointerFromNumero(k);

	    if ( ptr == NULL ) {
		result = false;
		break;	      
	    }

	    string elemLabel = ptr->getLabel();

	    beam2Moments transpMoments;

	    if (!moments2FromTransport(workingDir, elemLabel, ptr->getNomdElement(),transpMoments )  )
	      {
		result = false;
		break;
	      }
	    beams.back().set2Moments(transpMoments);
	  }
	return result;
}


bool  softwareTransport::moments2FromTransport(string workingDir, string elLab, const nomdElements elem, beam2Moments& moments) const {

  string elementLabel = elLab;
  // transformer le label en majuscules ; je ne suis pas sur que ca 
  // marchera a tous les coups (glm)
  std::transform(elementLabel.begin(), elementLabel.end(), elementLabel.begin(), (int (*)(int))std::toupper);

  cout << " dataManager::moments2FromTransport on cherche element " << elementLabel << endl;
  ifstream infile;
  string nameIn = workingDir + "transport.output";
  infile.open(nameIn.c_str(), ios::in);
  if (!infile) {
    cerr << " dataManager::moments2FromTransport : error re-opening transport output stream " << nameIn << endl;
    return false;
  }
  //  else cout << " particleBeam::setFromTransport() : ouverture du fichier " << nameIn << endl; 

  string::size_type nn = string::npos;
  string  fichier;
  string buf;
  unsigned compteur = 0;
  while ( getline(infile, buf) ) {
      fichier += buf+"\n";
    nn = buf.find(elementLabel);
    if ( nn != string::npos ) {
      compteur++;
      //      break;
    }
  }
  infile.close();
  cout << " compteur= " << compteur << endl;
  if ( compteur == 0 ) {
    cerr << " dataManager::moments2FromTransport : element " << elementLabel << " non trouve dans le fichier  " << nameIn << endl;
    return false;
  }


  stringstream fichierStream(fichier);
  buf.clear();
  unsigned relu = 0;
  while ( std::getline(fichierStream, buf) ) {
    //    cout << " relecture buffer : " << buf << endl;
    nn = buf.find(elementLabel);
    if ( nn != string::npos ) {
      relu++;
      if ( relu == compteur ) {
	cout << " TROUVE !" << endl;
	break;
      }
    }
  }
  moments.readFromTransportOutput(fichierStream);
  //  impressionDesMoments();
  return true;

}

